%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2017 ROBOTIS CO., LTD.
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%     http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distributed on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Author: Ryu Woon Jung (Leon)

%
% *********     Read and Write Example      *********
%
%
% Available Dynamixel model on this example : All models using Protocol 2.0
% This example is designed for using a Dynamixel PRO 54-200, and an USB2DYNAMIXEL.
% To use another Dynamixel model, such as X series, see their details in E-Manual(emanual.robotis.com) and edit below variables yourself.
% Be sure that Dynamixel PRO properties are already set as %% ID : 1 / Baudnum : 1 (Baudrate : 57600)
%

clc;
clear all;

lib_name = '';

if strcmp(computer, 'PCWIN')
  lib_name = 'dxl_x86_c';
elseif strcmp(computer, 'PCWIN64')
  lib_name = 'dxl_x64_c';
elseif strcmp(computer, 'GLNX86')
  lib_name = 'libdxl_x86_c';
elseif strcmp(computer, 'GLNXA64')
  lib_name = 'libdxl_x64_c';
elseif strcmp(computer, 'MACI64')
  lib_name = 'libdxl_mac_c';
end

% Load Libraries
if ~libisloaded(lib_name)
    [notfound, warnings] = loadlibrary(lib_name, 'dynamixel_sdk.h', 'addheader', 'port_handler.h', 'addheader', 'packet_handler.h');
end

global KEY_IS_PRESSED
global PROTOCOL_VERSION;
global ADDR_PRO_TORQUE_ENABLE;
global ADDR_PRO_TORQUE_ENABLE;
global TORQUE_ENABLE;
global dxl_comm_result;
global dxl_error;
global COMM_SUCCESS;
global ADDR_PRO_GOAL_VELOCITY;
KEY_IS_PRESSED = 0;

gcf
set(gcf, 'KeyPressFcn', @myKeyPressFcn);

% Control table address
ADDR_PRO_TORQUE_ENABLE       = 64;         % Control table address is different in Dynamixel model
ADDR_PRO_GOAL_POSITION       = 116;
ADDR_PRO_PRESENT_POSITION    = 132;
ADDR_PRO_GOAL_VELOCITY       = 104;

% Protocol version

PROTOCOL_VERSION            = 2.0;          % See which protocol version is used in the Dynamixel

% Default setting
DXL_ID_1                      = 1;            % Dynamixel ID: 1
DXL_ID_2                      = 2;            % Dynamixel ID: 2
DXL_ID_3                      = 3;            % Dynamixel ID: 3
DXL_ID_4                      = 4;            % Dynamixel ID: 4
DXL_ID_5                      = 5;            % Dynamixel ID: 5
BAUDRATE                    = 1000000;
DEVICENAME                  = 'COM4';       % Check which port is being used on your controller
                                            % ex) Windows: 'COM1'   Linux: '/dev/ttyUSB0' Mac: '/dev/tty.usbserial-*'

TORQUE_ENABLE               = 1;            % Value for enabling the torque
TORQUE_DISABLE              = 0;            % Value for disabling the torque
DXL_MINIMUM_POSITION_VALUE  = 0;      % Dynamixel will rotate between this value
DXL_MAXIMUM_POSITION_VALUE  = 4095;       % and this value (note that the Dynamixel would not move when the position value is out of movable range. Check e-manual about the range of the Dynamixel you use.)
DXL_MOVING_STATUS_THRESHOLD = 20;           % Dynamixel moving status threshold

ESC_CHARACTER               = 'q';          % Key for escaping loop
w_CHARACTER                 = 'w';          % Key for escaping loop
s_CHARACTER                 = 's';          % Key for escaping loop



COMM_SUCCESS                = 0;            % Communication Success result value
COMM_TX_FAIL                = -1001;        % Communication Tx Failed

% Initialize PortHandler Structs
% Set the port path
% Get methods and members of PortHandlerLinux or PortHandlerWindows
global port_num ;

port_num = portHandler(DEVICENAME);

% Initialize PacketHandler Structs
packetHandler();

index = 2;
dxl_comm_result = COMM_TX_FAIL;           % Communication result
dxl_goal_position = [DXL_MINIMUM_POSITION_VALUE DXL_MAXIMUM_POSITION_VALUE];         % Goal position

dxl_error = 0;                              % Dynamixel error
dxl_present_position = 0;                   % Present position


% Open port
if (openPort(port_num))
    fprintf('Succeeded to open the port!\n');
else
    closePort(port_num);
    unloadlibrary(lib_name);
    fprintf('Failed to open the port!\n');
    input('Press any key to terminate...\n');
    return;
end


% Set port baudrate
if (setBaudRate(port_num, BAUDRATE))
    fprintf('Succeeded to change the baudrate!\n');
else
    fprintf('Failed to change the baudrate!\n');
    input('Press any key to terminate...\n');
    return;
end


% Enable Dynamixel Torque DXL_ID_1
enableTorque(DXL_ID_1)

% Enable Dynamixel Torque DXL_ID_2
write1ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID_2, ADDR_PRO_TORQUE_ENABLE, TORQUE_ENABLE);
dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
if dxl_comm_result ~= COMM_SUCCESS
    fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
elseif dxl_error ~= 0
    fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
else
    fprintf('Dynamixel has been successfully connected \n');
end

enableTorque(DXL_ID_3);
enableTorque(DXL_ID_4);

% Enable Dynamixel Torque DXL_ID_5
write1ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID_5, ADDR_PRO_TORQUE_ENABLE, TORQUE_ENABLE);
dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
if dxl_comm_result ~= COMM_SUCCESS
    fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
elseif dxl_error ~= 0
    fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
else
    fprintf('Dynamixel has been successfully connected \n');
end


while 1
    %w = waitforbuttonpress;
    %input1 = Current
    %inputChar = input('Press any key to continue! (or input e to quit!)\n', 's');
    pause(0.2);
    switch KEY_IS_PRESSED
        case 0;
            stopMotor(DXL_ID_2)
            stopMotor(DXL_ID_3)
            stopMotor(DXL_ID_4)
        case char('q');
            break;
        case char('w');
            moveMotor(DXL_ID_2, 30);
        case char('s');
            moveMotor(DXL_ID_2, -30);
        case char('e');
            moveMotor(DXL_ID_3, 12);
        case char('d');
            moveMotor(DXL_ID_3, -12);
        case char('r');
            moveMotor(DXL_ID_4, 12);
        case char('f');
            moveMotor(DXL_ID_4, -12);
        case char('1');
            % Write goal position
            write4ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID_1, ADDR_PRO_GOAL_POSITION, typecast(int32(dxl_goal_position(index)), 'uint32'));
            dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
            dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
            if dxl_comm_result ~= COMM_SUCCESS
                fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
            elseif dxl_error ~= 0
                fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
            end

            while 1
                % Read present position
                dxl_present_position = read4ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID_1, ADDR_PRO_PRESENT_POSITION);
                dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
                dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
                if dxl_comm_result ~= COMM_SUCCESS
                    fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
                elseif dxl_error ~= 0
                    fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
                end

                fprintf('[ID:%03d] GoalPos:%03d  PresPos:%03d\n', DXL_ID_1, dxl_goal_position(index), typecast(uint32(dxl_present_position), 'int32'));
                
                if ~(abs(dxl_goal_position(index) - typecast(uint32(dxl_present_position), 'int32')) > DXL_MOVING_STATUS_THRESHOLD)
                    break;
                end
                
            end
            % Change goal position
            if index == 1
                index = 2;
            else
                index = 1;
            end
        case char('5');
            % Write goal position
            write4ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID_5, ADDR_PRO_GOAL_POSITION, typecast(int32(dxl_goal_position(index)), 'uint32'));
            dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
            dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
            if dxl_comm_result ~= COMM_SUCCESS
                fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
            elseif dxl_error ~= 0
                fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
            end
            
            while 1
                % Read present position
                dxl_present_position = read4ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID_5, ADDR_PRO_PRESENT_POSITION);
                dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
                dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
                if dxl_comm_result ~= COMM_SUCCESS
                    fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
                elseif dxl_error ~= 0
                    fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
                end

                fprintf('[ID:%03d] GoalPos:%03d  PresPos:%03d\n', DXL_ID_5, dxl_goal_position(index), typecast(uint32(dxl_present_position), 'int32'));


                if ~(abs(dxl_goal_position(index) - typecast(uint32(dxl_present_position), 'int32')) > DXL_MOVING_STATUS_THRESHOLD)
                    break;
                end
                
            end
            % Change goal position
            if index == 1
                index = 2;
            else
                index = 1;
            end
            

    end
    
    KEY_IS_PRESSED = 0;

end

%{
disableTorque(DXL_ID_1);
disableTorque(DXL_ID_2);
disableTorque(DXL_ID_3);
disableTorque(DXL_ID_4);
disableTorque(DXL_ID_5);
%}

% Close port
closePort(port_num);

% Unload Library
unloadlibrary(lib_name);

close all;
clear all;

function disableTorque(DXL_ID)
    % Disable Dynamixel Torque
    global port_num;
    global PROTOCOL_VERSION;
    global ADDR_PRO_TORQUE_ENABLE;
    global TORQUE_ENABLE;
    global dxl_comm_result;
    global dxl_error;
    global COMM_SUCCESS;
    write1ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID, ADDR_PRO_TORQUE_ENABLE, TORQUE_DISABLE);
    dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
    dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
    if dxl_comm_result ~= COMM_SUCCESS
        fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
    elseif dxl_error ~= 0
        fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
    end
end

function moveMotor(DXL_ID, velocity)
    % Write goal position
    global port_num;
    global PROTOCOL_VERSION;
    global ADDR_PRO_TORQUE_ENABLE;
    global TORQUE_ENABLE;
    global dxl_comm_result;
    global dxl_error;
    global COMM_SUCCESS;
    global ADDR_PRO_GOAL_VELOCITY;
    write4ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID, ADDR_PRO_GOAL_VELOCITY, typecast(int32(velocity), 'uint32'));
    dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
    dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
    if dxl_comm_result ~= COMM_SUCCESS
        fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
    elseif dxl_error ~= 0
        fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
    end
end

function stopMotor(DXL_ID)
    %disp('Nullify');
    % Write goal position
    global port_num;
    global PROTOCOL_VERSION;
    global ADDR_PRO_TORQUE_ENABLE;
    global TORQUE_ENABLE;
    global dxl_comm_result;
    global dxl_error;
    global COMM_SUCCESS;
    global ADDR_PRO_GOAL_VELOCITY;
    write4ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID, ADDR_PRO_GOAL_VELOCITY, 0);
    dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
    dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
    if dxl_comm_result ~= COMM_SUCCESS
        fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
    elseif dxl_error ~= 0
        fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
    end
end

function enableTorque(DXL_ID)
    global port_num;
    global PROTOCOL_VERSION;
    global ADDR_PRO_TORQUE_ENABLE;
    global TORQUE_ENABLE;
    global dxl_comm_result;
    global dxl_error;
    global COMM_SUCCESS;
    write1ByteTxRx(port_num, PROTOCOL_VERSION, DXL_ID, ADDR_PRO_TORQUE_ENABLE, TORQUE_ENABLE);
    dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION);
    dxl_error = getLastRxPacketError(port_num, PROTOCOL_VERSION);
    if dxl_comm_result ~= COMM_SUCCESS
        fprintf('%s\n', getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));
    elseif dxl_error ~= 0
        fprintf('%s\n', getRxPacketError(PROTOCOL_VERSION, dxl_error));
    else
        fprintf('Dynamixel has been successfully connected \n');
    end
end

function myKeyPressFcn(hObject, event)
    global KEY_IS_PRESSED
    KEY_IS_PRESSED  = get(gcf,'CurrentCharacter');
    disp(KEY_IS_PRESSED)
    disp('key is pressed')
end
