% if defined(__linux__) || defined(__APPLE__)
% include <fcntl.h>
% include <termios.h>
% define STDIN_FILENO 0
% elif defined(_WIN32) || defined(_WIN64)
% include <conio.h>
% endif
% 
% include <stdlib.h>
% include <stdio.h>
% include <time.h>
% include <errno.h>
% include "dynamixel_sdk.h"                                  // Uses Dynamixel SDK library

% Control table address
define ADDR_PRO_TORQUE_ENABLE          64                 // Control table address is different in Dynamixel model
define ADDR_PRO_GOAL_POSITION          116
define ADDR_PRO_PRESENT_POSITION       132

% Data Byte Length
define LEN_PRO_GOAL_POSITION           4
define LEN_PRO_PRESENT_POSITION        4

% Protocol version
define PROTOCOL_VERSION                2.0                 // See which protocol version is used in the Dynamixel

% Default setting
define BAUDRATE                        1000000
define DEVICENAME                      "/dev/ttyUSB0"      // Check which port is being used on your controller
                                                            // ex) Windows: "COM1"   Linux: "/dev/ttyUSB0" Mac: "/dev/tty.usbserial-*"

struct timespec tstart, tend;

int msleep(long msec)
{
    struct timespec ts;
    int res;

    if (msec < 0)
    {
        errno = EINVAL;
        return -1;
    }

    ts.tv_sec = msec / 1000;
    ts.tv_nsec = (msec % 1000) * 1000000;

    do {
        res = nanosleep(&ts, &ts);
    } while (res && errno == EINTR);

    return res;
}

int64_t timespecDiff(struct timespec *timeA_p, struct timespec *timeB_p)
{
  return ((timeA_p->tv_sec * 1000000000) + timeA_p->tv_nsec) -
           ((timeB_p->tv_sec * 1000000000) + timeB_p->tv_nsec);
}

int main(int argc, char **argv)
{
  int delay_value = 0;
  if (argc < 2) {
    printf("Usage : ./sync_read_write <delay_in_MS>\n");
    return 0;
  }

  delay_value = atoi(argv[1]);
  % Initialize PortHandler Structs
  % Set the port path
  % Get methods and members of PortHandlerLinux or PortHandlerWindows
  int port_num = portHandler(DEVICENAME);

  % Initialize PacketHandler Structs
  packetHandler();

  % Initialize Groupsyncread Structs for Present Position
  int groupread_num = groupSyncRead(port_num, PROTOCOL_VERSION, ADDR_PRO_PRESENT_POSITION, LEN_PRO_PRESENT_POSITION);

  int dxl_comm_result = COMM_TX_FAIL;               // Communication result
  uint8_t dxl_addparam_result = False;              // AddParam result
  uint8_t dxl_getdata_result = False;               // GetParam result

  uint8_t dxl_id[5] = {1, 2, 3, 4, 5};
  int32_t dxl_present_position[5] = {0};                           // Present position

  % Open port
  if (openPort(port_num))
  {
    printf("Succeeded to open the port!\n");
  }
  else
  {
    printf("Failed to open the port!\n");
    return 0;
  }

  % Set port baudrate
  if (setBaudRate(port_num, BAUDRATE))
  {
    printf("Succeeded to change the baudrate!\n");
  }
  else
  {
    printf("Failed to change the baudrate!\n");
    return 0;
  }

  % Add parameter storage for Dynamixel#1 present position value
  for(int i = 0; i < 5; ) {
    dxl_addparam_result = groupSyncReadAddParam(groupread_num, dxl_id[i]);
    if (dxl_addparam_result != True) {
      fprintf(stderr, "[ID:%03d] groupSyncRead addparam failed\n", dxl_id[i]);
      return 0;
    }
    else {
      printf("[ID:%03d] groupSyncRead addparam Succeeded\n", dxl_id[i]);
    }
    i++;
  }

  while (1)
  {
    clock_gettime(CLOCK_MONOTONIC, &tstart);
    % Syncread present position
    groupSyncReadTxRxPacket(groupread_num);
    if ((dxl_comm_result = getLastTxRxResult(port_num, PROTOCOL_VERSION)) != COMM_SUCCESS)
      printf("%s\n", getTxRxResult(PROTOCOL_VERSION, dxl_comm_result));

    % Check if groupsyncread data of Dynamixel#1 is available
    for (int j = 0; j < 5; ) 
        {
      dxl_getdata_result = groupSyncReadIsAvailable(groupread_num, dxl_id[j], ADDR_PRO_PRESENT_POSITION, LEN_PRO_PRESENT_POSITION);
      
      if (dxl_getdata_result != True) {
        fprintf(stderr, "[ID:%03d] groupSyncRead getdata failed", dxl_id[j]);
        return 0;
      }
      else {
        dxl_present_position[j] = groupSyncReadGetData(groupread_num, dxl_id[j], ADDR_PRO_PRESENT_POSITION, LEN_PRO_PRESENT_POSITION);
        printf("[ID:%03d] :%03d  ", dxl_id[j], dxl_present_position[j]);
      }
      j++;
    }
    printf("\n");
    msleep(delay_value);
    clock_gettime(CLOCK_MONOTONIC, &tend);
    printf("[%ld ms] ", timespecDiff(&tend, &tstart)/1000000);
  }

  % Close port
  closePort(port_num);

  return 0;
};
