% clc;
% clear all;

lib_name = '';

if strcmp(computer, 'PCWIN')
  lib_name = 'dxl_x86_c';
elseif strcmp(computer, 'PCWIN64')
  lib_name = 'dxl_x64_c';
elseif strcmp(computer, 'GLNX86')
  lib_name = 'libdxl_x86_c';
elseif strcmp(computer, 'GLNXA64')
  lib_name = 'libdxl_x64_c';
elseif strcmp(computer, 'MACI64')
  lib_name = 'libdxl_mac_c';
end

% Load Libraries
if ~libisloaded(lib_name)
    [notfound, warnings] = loadlibrary(lib_name, 'dynamixel_sdk.h', 'addheader', 'port_handler.h', 'addheader', 'packet_handler.h');
end





% gcf
% set(gcf, 'KeyPressFcn', @myKeyPressFcn);

% Control table address
ADDR_PRO_TORQUE_ENABLE       = 64;         % Control table address is different in Dynamixel model
ADDR_PRO_GOAL_POSITION       = 116;
ADDR_PRO_PRESENT_POSITION    = 132;
ADDR_PRO_GOAL_VELOCITY       = 104;
ADDR_PRO_PRESENT_VELOCITY    = 128;
ADDR_PRO_GOAL_CURRENT        = 102;
ADDR_PRO_PRESENT_CURRENT     = 126;
ADDR_PRO_GOAL_PWM            = 100;
ADDR_PRO_PRESENT_PWM         = 124;

% Protocol version

PROTOCOL_VERSION            = 2.0;          % See which protocol version is used in the Dynamixel

% Default setting
DXL_ID_1                      = 1;            % Dynamixel ID: 1
DXL_ID_2                      = 2;            % Dynamixel ID: 2
DXL_ID_3                      = 3;            % Dynamixel ID: 3
DXL_ID_4                      = 4;            % Dynamixel ID: 4
DXL_ID_5                      = 5;            % Dynamixel ID: 5
DXL_ID_6                      = 6;            % Dynamixel ID: 1
DXL_ID_7                      = 7;            % Dynamixel ID: 2
DXL_ID_8                      = 8;            % Dynamixel ID: 3
DXL_ID_9                      = 9;            % Dynamixel ID: 4
DXL_ID_10                     = 10;            % Dynamixel ID: 5
DXL_ID_11                     = 11;            % Dynamixel ID: 4
DXL_ID_12                     = 12;
BAUDRATE                    = 1000000;
DEVICENAME                  = 'COM4';       % Check which port is being used on your controller
DEVICENAME2                 = 'COM5';       % ex) Windows: 'COM1'   Linux: '/dev/ttyUSB0' Mac: '/dev/tty.usbserial-*'

TORQUE_ENABLE               = 1;            % Value for enabling the torque
TORQUE_DISABLE              = 0;            % Value for disabling the torque
DXL_MINIMUM_POSITION_VALUE  = 0;      % Dynamixel will rotate between this value
DXL_MAXIMUM_POSITION_VALUE  = 4095;       % and this value (note that the Dynamixel would not move when the position value is out of movable range. Check e-manual about the range of the Dynamixel you use.)
DXL_MOVING_STATUS_THRESHOLD = 20;           % Dynamixel moving status threshold

ESC_CHARACTER               = 'q';          % Key for escaping loop
w_CHARACTER                 = 'w';          % Key for escaping loop
s_CHARACTER                 = 's';          % Key for escaping loop



COMM_SUCCESS                = 0;            % Communication Success result value
COMM_TX_FAIL                = -1001;        % Communication Tx Failed

% Initialize PortHandler Structs
% Set the port path
% Get methods and members of PortHandlerLinux or PortHandlerWindows
% global port_num ;

port_num = portHandler(DEVICENAME);
port_num2 = portHandler(DEVICENAME2);

% Initialize PacketHandler Structs
packetHandler();

index = 2;
dxl_comm_result = COMM_TX_FAIL;           % Communication result
dxl_goal_position = [DXL_MINIMUM_POSITION_VALUE DXL_MAXIMUM_POSITION_VALUE];  % Goal position

dxl_error = 0;                              % Dynamixel error
dxl_present_position = 0;                   % Present position


% Open port
if (openPort(port_num))
    fprintf('Succeeded to open the port!\n');
else
    closePort(port_num);
    unloadlibrary(lib_name);
    fprintf('Failed to open the port!\n');
    input('Press any key to terminate...\n');
    return;
end

% Open port2
if (openPort(port_num2))
    fprintf('Succeeded to open the port!\n');
else
    unloadlibrary(lib_name);
    fprintf('Failed to open the port!\n');
    input('Press any key to terminate...\n');
    return;
end


% Set port baudrate
if (setBaudRate(port_num, BAUDRATE))
    fprintf('Succeeded to change the baudrate!\n');
else
    fprintf('Failed to change the baudrate!\n');
    input('Press any key to terminate...\n');
    return;
end

% Set port2 baudrate
if (setBaudRate(port_num2, BAUDRATE))
    fprintf('Succeeded to change the baudrate!\n');
else
    unloadlibrary(lib_name);
    fprintf('Failed to change the baudrate!\n');
    input('Press any key to terminate...\n');
    return;
end

% Disable Dynamixel Torque for DXL_ID_1 to DXL_ID_6 for robot 1
% Enable Dynamixel Torque DXL_ID_7 to 11 for robot 2
enableTorque(port_num,DXL_ID_1,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_DISABLE,COMM_SUCCESS);
enableTorque(port_num,DXL_ID_2,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_DISABLE,COMM_SUCCESS);
enableTorque(port_num,DXL_ID_3,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_DISABLE,COMM_SUCCESS);
enableTorque(port_num,DXL_ID_4,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_DISABLE,COMM_SUCCESS);
enableTorque(port_num,DXL_ID_5,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_DISABLE,COMM_SUCCESS);
enableTorque(port_num,DXL_ID_6,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_DISABLE,COMM_SUCCESS);

enableTorque(port_num2,DXL_ID_7,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_ENABLE,COMM_SUCCESS);
enableTorque(port_num2,DXL_ID_8,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_ENABLE,COMM_SUCCESS);
enableTorque(port_num2,DXL_ID_9,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_ENABLE,COMM_SUCCESS);
enableTorque(port_num2,DXL_ID_10,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_ENABLE,COMM_SUCCESS);
enableTorque(port_num2,DXL_ID_11,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_ENABLE,COMM_SUCCESS);
enableTorque(port_num2,DXL_ID_12,PROTOCOL_VERSION,ADDR_PRO_TORQUE_ENABLE,TORQUE_ENABLE,COMM_SUCCESS);



